;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x8e8c4f694be8d069afe81dd191afef28 . "./code.cg2js/cg-acache.fasl")
(#xfc7007536b48a63b1018668d427611ff . "./code.cg2js/cg-base.fasl")
(#x276bfdfc6f1d9ce960ef55b72f7e8532 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x8174412b5d0386d81d80c62b1e4bd592 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x052d424993f7a5ca11c75304d8d2faba . "./code.cg2js/cg-bitmap-stream.fasl")
(#x60a4e692f141612ae023774468dd216d . "./code.cg2js/cg-button.fasl")
(#xa1a842603bd65c3b19cdb57abbd7b16d . "./code.cg2js/cg-calendar.fasl")
(#xc792e57a5ac24c84dd8e65ca2ea25f9e . "./code.cg2js/cg-caret.fasl")
(#x9159360b2b969242aef82b6a0d81b8a5 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x56e5349ec6eac10dec32cac78f1f386c . "./code.cg2js/cg-chart-widget.fasl")
(#x23c0044b8314d2fa3e11035c05455e52 . "./code.cg2js/cg-check-box.fasl")
(#xa4c0f930ba4f75ed5a9301812a353776 . "./code.cg2js/cg-choice-list.fasl")
(#x148bcc66aa79582d99952fd377c4934f . "./code.cg2js/cg-choose-printer.fasl")
(#xd604af48a97624480b9e979fb34d6fff . "./code.cg2js/cg-class-grid.fasl")
(#x0c4b1deddcbf891090b6ecfa6c308343 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x84734aad4e1c76683ae5e71b90209296 . "./code.cg2js/cg-class-support.fasl")
(#xe801d8c1e3dee8d93bed5af8e2a93ca2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc8d7d574b3e492b5185f406b79a323af . "./code.cg2js/cg-clipboard-stack.fasl")
(#x384ee731bf54c9143f48b1554619cbb7 . "./code.cg2js/cg-clipboard.fasl")
(#xbfcc62950711d36ad9b272b73d1f2281 . "./code.cg2js/cg-color-dialog.fasl")
(#xa7e838f2fa3781eba7b0cc83093f67ee . "./code.cg2js/cg-combo-box.fasl")
(#x37b464e77cfe6e6b0a6be52ced165fb5 . "./code.cg2js/cg-common-control.fasl")
(#xfc26939254842e96d0b1f7a924cb0452 . "./code.cg2js/cg-comtab.fasl")
(#xd33dbb3d874ed1c8283f61f61162060d . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x39855202805533089fe9f73333fc9507 . "./code.cg2js/cg-curve.fasl")
(#x01554852354ef23c24b7c60d55101fce . "./code.cg2js/cg-dde-utils.fasl")
(#xb59c3d7d945c866d3867703e60ab7dae . "./code.cg2js/cg-dev.fasl")
(#x3574be3800402e07d6e6e37b02c13991 . "./code.cg2js/cg-dialog-item.fasl")
(#xcdf76a64e11638cdaaa9dc30b0cee4c1 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1d21581ab4d3a6e3e6e167fe4bf1474a . "./code.cg2js/cg-directory-dialog.fasl")
(#xe53fdd0e68c74c983e0e9942a9585b06 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x48b4d789f1417776620beb6c02599c8c . "./code.cg2js/cg-drag-and-drop.fasl")
(#xf7cbbaa6ad9d6b1f84493488c33ef12f . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xdecdd8d8cb1a487983f73664818218e1 . "./code.cg2js/cg-drawable.fasl")
(#x281651ba6e4a111f8cf1e1a57b038358 . "./code.cg2js/cg-dropping-outline.fasl")
(#x8df80e824388f9fd7a5b33ee95fe8a43 . "./code.cg2js/cg-edit-in-place.fasl")
(#xde27fb569e21a09c080c4da5b048e095 . "./code.cg2js/cg-editable-text.fasl")
(#x9a2ece6c0f6bbc41ebee89c8420dc034 . "./code.cg2js/cg-file-dialog.fasl")
(#xacc4bcb22ba89429d623bb9dc1492ba6 . "./code.cg2js/cg-fill-texture.fasl")
(#xfc797755066dc63a44236a29d3b9ee99 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe036d46358171a758f7558eae4c8692f . "./code.cg2js/cg-font-dialog.fasl")
(#xec76109ecdc702288a30af30749754da . "./code.cg2js/cg-gesture-emulation.fasl")
(#xa7545fb203829fd02d0433eac6b58693 . "./code.cg2js/cg-get-pixmap.fasl")
(#x8d2f45397e586280d323864b9bd6aee8 . "./code.cg2js/cg-get-position.fasl")
(#xe5d5f9d63370e50f1ac864b6516b8b4d . "./code.cg2js/cg-graphics-context.fasl")
(#x01caabdb3703fb9144a1acfe850a9661 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xa12940122c19ac673631934ab0bffccc . "./code.cg2js/cg-grid-widget.fasl")
(#xb1e072c36f54bbfe01fa38b08035c0a8 . "./code.cg2js/cg-group-box.fasl")
(#xdb13079b9b705f53dccf9341cc7fd895 . "./code.cg2js/cg-header-control.fasl")
(#x5bf6c123502cff9ba1bd7a7460a8f0a2 . "./code.cg2js/cg-hotspot.fasl")
(#x3a68b93b244b444448268733a4b08ffc . "./code.cg2js/cg-html-dialog.fasl")
(#x7a613f95c686531e480935b327e92b3d . "./code.cg2js/cg-html-widget.fasl")
(#xff97eb5b183c697ff110a702b2cefbf6 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xc419297abb771ffab7d1fc1e73cd7d0e . "./code.cg2js/cg-icon.fasl")
(#x2784ab977bdc1e18f809e2cc61f2113a . "./code.cg2js/cg-ie.fasl")
(#x035b85fdf783f89cfe63a000ba5f044c . "./code.cg2js/cg-intersect-posbox.fasl")
(#xa6bb7c637b19a33a072edf604240a8ba . "./code.cg2js/cg-intersect.fasl")
(#xf0c43f2d61a278c7cd1140212af890b6 . "./code.cg2js/cg-item-list.fasl")
(#x1d71122b2d9afcd5c93cc764584638eb . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x04ecfb46c4f5ec67dfdab39748ea5096 . "./code.cg2js/cg-lamp.fasl")
(#x9715506d70786cb1c6d8c16276034c2f . "./code.cg2js/cg-layout.fasl")
(#x03299f0948744fe9bfc189583a1c9ece . "./code.cg2js/cg-lettered-menu.fasl")
(#x592c611036eb452a83496fc5488546c5 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xb60b46b53d0603b4b0a8aa777f512060 . "./code.cg2js/cg-lisp-text.fasl")
(#xadf68d95c68d1f347cf5379ad40fef89 . "./code.cg2js/cg-lisp-widget.fasl")
(#x35cec316272cbcc67d4986a720d8e200 . "./code.cg2js/cg-list-view.fasl")
(#xd6533bb0c70c7fb8743733b15d2b6889 . "./code.cg2js/cg-mci.fasl")
(#x213bbb31d4c6a885b39e7da52fa17145 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x485be16770fa262c628e673b06ba78a0 . "./code.cg2js/cg-menu.fasl")
(#xbdeaeb56fb410e848150262ce19ba234 . "./code.cg2js/cg-message-dialog.fasl")
(#x340ac7bc6aee936e6a10809ead21bb56 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x7f40eede8eff7260d511204c9f3eb2e0 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xba0f455eecb2fd1f3af7c63696467eee . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x9e81f36aea34303ef8dd7f18da283da0 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x013d27d923c94291fb796de158badd76 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc9c54faa2b9cde1ef7836a76b3cb9ba3 . "./code.cg2js/cg-nodes.fasl")
(#x826509e9b37d366aaa19fbede5d771d4 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x0cf37efe3c880068fe6ba2046fc17ee0 . "./code.cg2js/cg-object-editor.fasl")
(#x1cea596adc098266010a6bdf87af6494 . "./code.cg2js/cg-ocx.fasl")
(#x4f06080b6c8f2ca24c1b9e6d2e3930d0 . "./code.cg2js/cg-os-widget.fasl")
(#xc1a853f3e5f3dc9f9e00be6854ac0d34 . "./code.cg2js/cg-os-window.fasl")
(#xebd756743b0f7506ee7c77dcd76e65ee . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x661b842731fa94f9922cbb86e5edcd43 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xbeb297f6b89c9644dd4ce88216d9b72f . "./code.cg2js/cg-outline.fasl")
(#x4085297fdd6958be08f757f0bd57e58f . "./code.cg2js/cg-palette.fasl")
(#x347097a80234460b61aec4226794fc4e . "./code.cg2js/cg-paren-matching.fasl")
(#x8424d2bd8a015294087b16f1e99a9052 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x63d0cefc6934eb1a5e85f4c3119ee8b0 . "./code.cg2js/cg-picture-widget.fasl")
(#xd5618fdd2bd9e2781c1f98b0be1da131 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xf87267b862cbd335d95bd34aa2fa3e68 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd393833be7428c1d2b9709a8d5c4e0d7 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x9ba8cdf44227e250b1a9437fda76f5e6 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa9b096f99dd9b52379cea90faa4a6b7d . "./code.cg2js/cg-pixmap.fasl")
(#xa33683379e77fb0835fbafeeb32f8620 . "./code.cg2js/cg-plot-widget.fasl")
(#xf96853a79b2aab38cc8311d71725a708 . "./code.cg2js/cg-printing.fasl")
(#x2aeb9359daa4efc300964b5624f77042 . "./code.cg2js/cg-progress-indicator.fasl")
(#xcf7b3bff33d0fa8a41c92c51f4b3e17f . "./code.cg2js/cg-project-window.fasl")
(#xbf0f12172d64c9a19c493ec7cbf0d601 . "./code.cg2js/cg-property.fasl")
(#x5e563345a68ab99328a9691d2d1ef7ac . "./code.cg2js/cg-radio-button.fasl")
(#xdbe0eae58888f167343307ebd337d713 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xdfee29fc9dbeb96bac50fe46db5d412b . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x2233ffad1700871599f1e627f63c7dd7 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x51e9ad016deabd20e00b54f3dfe895e6 . "./code.cg2js/cg-rich-edit.fasl")
(#xf5d55a400d37e604616e805535990092 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xdcc1e5060e75becfccf99050d761317a . "./code.cg2js/cg-scaling-stream.fasl")
(#xe1bc0f95a6db23858f55b400a95f6c63 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x2ed7282fb32cacb9c0eafb1c78c35b04 . "./code.cg2js/cg-scroll-bar.fasl")
(#xd9eea822a5080b6623c514e02b0bac64 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x6c789ac42e08e2ffa2e9c76f51c9d014 . "./code.cg2js/cg-selected-object.fasl")
(#xdbcf5274d445358a207c797c5d1e2ab8 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x36628bf5ecf5870da6adf3aaee7e3686 . "./code.cg2js/cg-split-bar.fasl")
(#x09df8db16f920ef61612fb0db1647835 . "./code.cg2js/cg-static-text.fasl")
(#xdb84203d71ed211e04a2a14e9b55acb6 . "./code.cg2js/cg-status-bar.fasl")
(#x82d0dbb17f41e1bdfe5444495dd3df29 . "./code.cg2js/cg-string-dialog.fasl")
(#x03a697305ec6c079f5e95183861ac2b3 . "./code.cg2js/cg-tab-control.fasl")
(#x6c87ec8b4ed9bb89cde911cc2833358d . "./code.cg2js/cg-template-string.fasl")
(#xac4a9b167cc935b115251886a7612da2 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xabfcfe38eb334521987519b051bd5809 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x2d235830f1d855c9f79290a6c1d5acfb . "./code.cg2js/cg-text-edit-pane.fasl")
(#x3bb3b8ae4487e517c2fb8566658148c3 . "./code.cg2js/cg-text-or-combo.fasl")
(#x0e6df04fb55fa69df39e750743ab2382 . "./code.cg2js/cg-text-widget.fasl")
(#x7cc778d4eab33a5f4e87a668c522b1a8 . "./code.cg2js/cg-timer.fasl")
(#x7abe421f5ca8369cf15af89ac2c57d32 . "./code.cg2js/cg-toggling-widget.fasl")
(#x58f8cb79336b6cd8891e1bbd65654a22 . "./code.cg2js/cg-toolbar.fasl")
(#xc193893b6646ae87afca8ba04350fcbe . "./code.cg2js/cg-tooltip.fasl")
(#xdef7956d50b90e6af535764f6c166a52 . "./code.cg2js/cg-touch.fasl")
(#x47f712c2b8eef60bd513d7cb2cfbc219 . "./code.cg2js/cg-trackbar.fasl")
(#x504a4bc20d3b4d38ed033f206454df27 . "./code.cg2js/cg-tray.fasl")
(#x6737894d7d9353b3f0d17d91a26838d8 . "./code.cg2js/cg-up-down-control.fasl")
(#xa791609212b06be4ecd6cf2134866051 . "./code.cg2js/cg-utility-dialog.fasl")
(#x9a2db37d57832a0cd905b2abdf62e84c . "./code.cg2js/cg-web-browser-dde.fasl")
(#xadb41211a71869f852473b6483bdc5f1 . "./code.cg2js/cg-web-browser.fasl")
(#x31e4508ecef61e8c7a2198f9e5d20329 . "./code.cg2js/cg-wrap-string.fasl")
(#xe5d7ce05992a7a4b7918972b9b90b428 . "./code.cg2js/cg-yes-no-list.fasl")
(#x827fae86b5425be596d96c1222ff6c39 . "./code.cg2js/cg-yes-no-string.fasl")
(#x2ed1d87feaf0d3538752c89d3495aaa6 . "./code.cg2js/cg.fasl")
(#xe2df3001aeed0f0fb08fe3376fee843e . "./code.idejs/ide-acache.fasl")
(#x52479a1814e10db14e795e1e5df35bc9 . "./code.idejs/ide-apropos.fasl")
(#x8faa2ad90e5f77660d2600f0a6eeb95c . "./code.idejs/ide-base.fasl")
(#x8bc7846ef35cbd05b544bf4e775a1d83 . "./code.idejs/ide-class-browser.fasl")
(#x8f2225e80df33a731c3acf43b147fcab . "./code.idejs/ide-class-grapher.fasl")
(#xfa73fac9f703524faf700859ec67c51f . "./code.idejs/ide-clipboard.fasl")
(#xa71715b376061540c7b02561a0f79f70 . "./code.idejs/ide-dev.fasl")
(#xdafba81dc9bcaf0d56856c1466104ee9 . "./code.idejs/ide-editor.fasl")
(#x728e3abbe486984ff47428da8eed64f4 . "./code.idejs/ide-find-definitions.fasl")
(#x9c848fc98562bad1bcdaafcd0da7c222 . "./code.idejs/ide-find-in-files.fasl")
(#xd0944833c5c4f3a428b9dfe2ba29955d . "./code.idejs/ide-form.fasl")
(#xfcd836450f62fa3ed41c0abb6af0d323 . "./code.idejs/ide-grapher.fasl")
(#x9d2e3de7ead317a53266fca1aad84f45 . "./code.idejs/ide-inspector.fasl")
(#x49d7b4dbf93a2c335cd420f53a2b79c7 . "./code.idejs/ide-intro.fasl")
(#x8f40c12b77f8a2d324d69a2f9ce9289d . "./code.idejs/ide-keyshort.fasl")
(#x1f64612404b4976697b812e1865de696 . "./code.idejs/ide-navigator.fasl")
(#x676c616e0e7df134a2925ee6db6bdeda . "./code.idejs/ide-package-list.fasl")
(#x14bc0b4a0982764343e3a9590da8e06a . "./code.idejs/ide-patch-dialog.fasl")
(#x7271b78b201224bdbdaadb80f5612afc . "./code.idejs/ide-process-browser.fasl")
(#x4b59c994ae23db60202ce645023c6a77 . "./code.idejs/ide-project.fasl")
(#x1f6801ed32532fd05e29ac475cfb1583 . "./code.idejs/ide-trace.fasl")
(#x7504934bb3afb4909dbdd11e25c30def . "./code.idejs/ide-window-list.fasl")
(#x48186a163f109f16c8e8ef6f16f81020 . "./code.idejs/ide.fasl")
(#xea5d13ce0f9a2cfde1ff539415770ab2 . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
